(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ContainedReact = {}, global.React));
})(this, (function (exports, react) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    /**
     * This converts an object or array into a primitive key.
     * This is useful for React hooks when you need a primitive key for the dependency array.
     *
     * We're using `JSON.stringify` to convert the object into a string.
     * This means that an reordering of the object or array will result in a different key.
     *
     * NOTE: Only do performance optimization when needed.
     *
     * @author @mthines
     *
     * @version 1.0.0
     *
     * @example
     * ```tsx
      const Component = () => {
        const key = useMemoKey(['a', 'b', 'c']); // <- This returns `["a","b","c"]`

        useEffect(() => {
          // Here is some heavy operation which should only execute when the `key` updates.
        }, [key])
      }
     * ```
     */

    var useMemoKey = function useMemoKey(item) {
      return react.useMemo(function () {
        if (typeof item === 'string' || typeof item === 'number' || typeof item === 'boolean' || item === null || item === undefined) {
          return item;
        }

        return JSON.stringify(item);
      }, [item]);
    };

    var isServer = typeof window === 'undefined';

    var getCalculatedSize = function getCalculatedSize(query) {
      var _a = query.split(':'),
          calculator = _a[0],
          size = _a[1];

      return {
        type: calculator,
        size: parseInt(size, 10)
      };
    };

    var useContainerQuery = function useContainerQuery(query, options) {
      var _a;

      var ref = react.useRef(null);

      var _b = react.useState(null),
          matches = _b[0],
          setMatches = _b[1];

      react.useEffect(function () {
        var _a;

        if (isServer || !ref.current) return;
        var observer = new ResizeObserver(function (_a) {
          var entry = _a[0];

          var _b = getCalculatedSize(query),
              size = _b.size,
              type = _b.type;

          if (type === 'min-width') {
            if (entry.contentRect.width >= size) {
              setMatches(true);
            } else {
              setMatches(false);
            }

            return;
          }

          if (type === 'max-width') {
            if (entry.contentRect.width <= size) {
              setMatches(true);
            } else {
              setMatches(false);
            }

            return;
          }

          if (type === 'min-height') {
            if (entry.contentRect.height >= size) {
              setMatches(true);
            } else {
              setMatches(false);
            }

            return;
          }

          if (type === 'max-height') {
            if (entry.contentRect.height <= size) {
              setMatches(true);
            } else {
              setMatches(false);
            }

            return;
          }
        });
        observer.observe(ref.current, {
          box: ((_a = options === null || options === void 0 ? void 0 : options.observeOptions) === null || _a === void 0 ? void 0 : _a.box) || 'border-box'
        });
        return function () {
          return observer.disconnect();
        };
      }, [(_a = options === null || options === void 0 ? void 0 : options.observeOptions) === null || _a === void 0 ? void 0 : _a.box, query]);
      return [ref, matches];
    };
    /**
     * Detecting multiple container queries within the same Element
     */

    var useContainerQueries = function useContainerQueries(queries, options) {
      var _a;

      var ref = react.useRef(null);

      var _b = react.useState({}),
          matches = _b[0],
          setMatches = _b[1];

      var queriesKey = useMemoKey(queries);
      react.useEffect(function () {
        var _a;

        if (isServer || !ref.current) return;
        var queriesEntries = Object.entries(queries);
        var observer = new ResizeObserver(function (_a) {
          var entry = _a[0];
          var matchingBreakpoints = queriesEntries.reduce(function (acc, _a) {
            var name = _a[0],
                query = _a[1];

            var _b = getCalculatedSize(query),
                size = _b.size,
                type = _b.type;

            if (type === 'min-width') {
              if (entry.contentRect.width >= size) {
                return __assign(__assign({}, acc), {
                  width: name
                });
              }
            }

            if (type === 'max-width') {
              if (entry.contentRect.width <= size) {
                return __assign(__assign({}, acc), {
                  width: name
                });
              }
            }

            if (type === 'min-height') {
              if (entry.contentRect.height >= size) {
                return __assign(__assign({}, acc), {
                  height: name
                });
              }
            }

            if (type === 'max-height') {
              if (entry.contentRect.height <= size) {
                return __assign(__assign({}, acc), {
                  height: name
                });
              }
            }

            return acc;
          }, {});

          if (Object.keys(matchingBreakpoints).length === 0) {
            setMatches({});
            return;
          }

          setMatches(matchingBreakpoints);
        });
        observer.observe(ref.current, {
          box: ((_a = options === null || options === void 0 ? void 0 : options.observeOptions) === null || _a === void 0 ? void 0 : _a.box) || 'border-box'
        });
        return function () {
          return observer.disconnect();
        }; // eslint-disable-next-line react-hooks/exhaustive-deps
      }, [(_a = options === null || options === void 0 ? void 0 : options.observeOptions) === null || _a === void 0 ? void 0 : _a.box, queriesKey]);
      return [ref, matches];
    };

    exports.useContainerQueries = useContainerQueries;
    exports.useContainerQuery = useContainerQuery;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
